#!/usr/bin/ruby
$:.unshift File.dirname(__FILE__)
require "confreader"
require "worker"
require "common"
require 'logger'

logger = Logger.new(File.join(File.dirname(__FILE__),"../logs/tiebraker_main.log"), 'weekly')
logger.datetime_format = "%Y-%m-%d %H:%M:%S"
logger.info "Started"

allvariables = ALLVARS.new
# config check
conf = CONFREADER.new(File.join(File.dirname(__FILE__), '../config.json'));
if ! conf.error_in_parsing_json.empty? then
	logger.fatal "Error at parsing json #{conf.error_in_parsing_json}"
	exit
end

conf.hashedjson["configs"].each {|x| 
	logger.info x
	if  allvariables.configisvlalid then
		if (x.has_key?("config_name") & x.has_key?("primary_ip") & x.has_key?("secondary_ip") & x.has_key?("primary_svm")   & x.has_key?("secondary_svm") & 
			x.has_key?("primary_user") & x.has_key?("primary_password") & x.has_key?("secondary_user") & x.has_key?("secondary_password") & 
			x.has_key?("primary_site_name") & x.has_key?("ips_to_check_on_primary_from_secondary") & x.has_key?("ips_to_check_on_primary") & 
			x.has_key?("ips_to_check_on_secondary") & x.has_key?("fc_topology_check") & x.has_key?("check_fc_switches_on_primary") &
			x.has_key?("primary_svm_user") & x.has_key?("primary_svm_password") & x.has_key?("secondary_svm_user") & x.has_key?("secondary_svm_password")
		    ) then
			logger.info "The config has all the keys."

			if ( (x["ips_to_check_on_primary"].class.name == 'Array') & (x["ips_to_check_on_secondary"].class.name == 'Array') & 
				(x["ips_to_check_on_primary_from_secondary"].class.name == 'Array') & (x["check_fc_switches_on_primary"].class.name == 'Array')) then
				logger.info "Found all the array type keys."
				if ( (x["ips_to_check_on_primary"].count>0) & (x["ips_to_check_on_secondary"].count>0) & (x["ips_to_check_on_primary_from_secondary"].count>0) &
					(  ( x["fc_topology_check"] & (x["check_fc_switches_on_primary"].count>0) ) ||  ! x["fc_topology_check"] )
					
					) then
					logger.info "The array types have more than zero elements in the config. It is Ok."
				else
					logger.fatal "There are keys with array type with no elements"
					allvariables.setconfigisvalid(false)
				end
			else
				logger.fatal "Some keys must be type of array!"
				allvariables.setconfigisvalid(false)
				
			end

		else
			logger.fatal "There are missing keys in the config."
			allvariables.setconfigisvalid(false)
		end

	end
} 

exit if !allvariables.configisvlalid


# start workers
threads = []
conf.hashedjson["configs"].each {|x| 
	logger.debug x
	threads << Thread.new { WORKER.new(x) }
} 


begin
	anythreadalive = false
	logger.info "Number of threads: #{threads.size}"
	threads.each {|x| 
		# exit if not x.alive?
 		anythreadalive = anythreadalive | x.alive?
 		x.exit if not x.alive?
	} 
	logger.info "Any thread alive: #{anythreadalive }"
	sleep 10
end while anythreadalive

