$:.unshift File.dirname(__FILE__)

require "ntap"
require 'net/ping'
require "common"
require 'logger'


class WORKER


	def initialize(config)
		@config = config
		
		@logger = Logger.new(File.join(File.dirname(__FILE__),"../logs/#{@config['config_name']}.log"), 'weekly')
		@logger.datetime_format = "%Y-%m-%d %H:%M:%S"
		@logger.info "Started"
		
		@allvars = ALLVARS.new
		@mtex = @allvars.mutex

		@checkofseparation = {}
		#@logger.info @allvars.isseparated
		@primary_vsm_properties = {"luns"=>[]}
		@secondary_vsm_properties = {"management_lifs"=>[],"data_lifs"=>[]}
		
		fillup_svm_details(true)
		
		@vsms_properties = {"primary_vsm_properties"=>@primary_vsm_properties, "secondary_vsm_properties"=>@secondary_vsm_properties}
		#puts @vsms_properties.inspect
		lun_scan_cycle = 0
		while true & (not @allvars.isseparated)
			@logger.info "In while"
			@checkofseparation = checkifseparated
			if   @checkofseparation["primary_is_alive"] | @checkofseparation["secondary_is_alive"]  then # ha nem separated
				if not @checkofseparation["primary_is_alive"] then # ha a primary nem erheto el a 3. site-rol, akkor megnezi, hogy a secondary-rol elerheto e a primary
					@logger.warn "#{@config['config_name']}: Primary site is not alive from 3. site check wheter netapp on secondary site is able to reach"
					netappinst = NETAPP.new(@config, @logger,@vsms_properties)
					if not netappinst.ping_primary_from_secondary then
						@logger.error "#{@config['config_name']}: Failover started..."
						if netappinst.failover then
						  @logger.info "#{@config_name}: Failover has been succeeded"
						  exit
						else
						  @logger.fatal "#{@config_name}: Failover has not been succeeded"
						  exit
						end
						
					end # do site failover
				end
			end # if ( @checkofseparation["primary_is_alive"] | @checkofseparation["secondary_is_alive"] )

			lun_scan_cycle+=1

			if lun_scan_cycle==5 then
				fillup_svm_details(false)
				lun_scan_cycle = 0
			end

			sleep 2
		end # while true & (not @allvars.isseparated
	end
	
	def checkifseparated
		# egyik ip sem ok. felteszem
		primary_is_alive = false
		secondary_is_alive = false
		
		# a primary ipk netan elerhetok
		@config["ips_to_check_on_primary"].each do |ip_to_check_on_primary|
			if not primary_is_alive then
				# primary_is_alive = Net::Ping::External.new(ip_to_check_on_primary).ping?
				primary_is_alive = do_ping(ip_to_check_on_primary,1)
				if primary_is_alive then 
					@logger.info "#{@config['config_name']}: The IP on primary site: #{ip_to_check_on_primary} is alive from 3. site"
				else
					@logger.warn "#{@config['config_name']}: The IP on primary site: #{ip_to_check_on_primary} is not alive from 3. site"
				end

			end
		end
		
		@config["ips_to_check_on_secondary"].each do |ip_to_check_on_secondary|
			if not secondary_is_alive then
				#secondary_is_alive = Net::Ping::External.new(ip_to_check_on_secondary).ping?
				secondary_is_alive = do_ping(ip_to_check_on_secondary,1)
				if secondary_is_alive then
					@logger.info "#{@config['config_name']}: The IP on secondary site: #{ip_to_check_on_secondary} is alive = #{secondary_is_alive} from 3. site"
				else
					@logger.warn "#{@config['config_name']}: The IP on secondary site: #{ip_to_check_on_secondary} is not alive = #{secondary_is_alive} from 3. site"
				end

			end
		end

		sitestate={"primary_is_alive"=>primary_is_alive, "secondary_is_alive"=>secondary_is_alive}

		if not primary_is_alive then # ha az elsodleges site nem megy, akkor megnezi, hogy a masodlagos megy e
				if not secondary_is_alive then # ha a masodlagos sem megy, akkor beallitja a setseparated-et true-ra
					@mtex.synchronize do
						if ! @allvars.isseparated then
							@logger.fatal "#{@config['config_name']}: Setting separated!!!!"
							@allvars.setseparated(true)
						end
					end
				else
					# dummy 
				end
			end
			@logger.info "#{@config['config_name']}: The state of sites: #{sitestate}"
		return sitestate # azert van return, hogy  1 forduloban lehessen is ellenorizni
	end 


	def do_ping(hostname,ping_count)
		pcount=0
		ping = Net::Ping::External.new(host = hostname,  timeout = 10, count=10)
		while !ping.ping? && pcount < ping_count
  			pcount+=1
		end
		return ping.ping?
	end # do_ping


	def svm_get_management_lifs(primary_or_secondary)
		netappinst = NETAPP.new(@config, @logger,{})
		return netappinst.get_management_interfaces(primary_or_secondary)
	end # first_check_health

	def svm_get_data_lifs(primary_or_secondary,protocol)
		netappinst = NETAPP.new(@config, @logger,{})
		return netappinst.get_data_interfaces(primary_or_secondary,protocol)
	end # first_check_health

	def svm_get_luns_primary
		netappinst = NETAPP.new(@config, @logger,{})
		return netappinst.get_primary_luns
	end # first_check_health

	def check_connection_to_netapp(destination,caller)
		netappinst = NETAPP.new(@config, @logger,{})
		return netappinst.test_connection(destination, caller)
	end # first_check_health


	def fillup_svm_details(isfirst)
		if isfirst then 
		
		###################
		# query manaement interfaces, trouble if no results

=begin for future use
			primary_svm_management_lifs = svm_get_management_lifs("primary")
			puts primary_svm_management_lifs.inspect
			if ! (primary_svm_management_lifs.class.name.eql?("NilClass") ) then
				if primary_svm_management_lifs.count > 0 then
					anymanagementinterfaceisup = false
					primary_svm_management_lifs.each { |vsm_management_interface|
				 		anymanagementinterfaceisup = true if vsm_management_interface["operational-status"].eql?("up")
					}
				if ! anymanagementinterfaceisup then
					@logger.fatal "#{@config_name}: No management interfaces have operational-status UP"
					exit
				end #! anymanagementinterfaceisup then
				else
					@logger.fatal "#{@config_name}: No management interfaces definied on primary SVM"
					exit
				end #if primary_svm_management_lifs.count == 0 then
			else #! (primary_svm_management_lifs.class.name.eql?("NilClass") ) then
				@logger.fatal "#{@config_name}: Unable to retrieve management interfaces from primary SVM"
				exit
			end
=end # for future use
			secondary_svm_management_lifs = svm_get_management_lifs("secondary")
			if ! (secondary_svm_management_lifs.class.name.eql?("NilClass") ) then
				if secondary_svm_management_lifs.count > 0 then
					any_management_interfaces_are_admin_up = false
					secondary_svm_management_lifs.each { |vsm_management_interface|
				 		any_management_interfaces_are_admin_up= true if vsm_management_interface["administrative-status"].eql?("up")
					}
					if ! any_management_interfaces_are_admin_up then
						@logger.fatal "#{@config_name}: No management interfaces have administrative-status UP on secondary"
						exit
					end #! any_management_interfaces_are_admin_up then
				else
					@logger.fatal "#{@config_name}: No management interfaces definied on secondary SVM"
					exit
				end
			else
				@logger.fatal "#{@config_name}: Unable to retrieve management interfaces of secondary SVM"
			end
			@secondary_vsm_properties["management_lifs"] =  secondary_svm_management_lifs
		# / query management interfaces, trouble if no results
		
		# query data interfaces, no trouble if no results		

=begin	# for future use
		primary_svm_data_lifs = []
		data_protocols = ["nfs","cifs","iscsi","fcp","fcache"]
		data_protocols.each{ |protocol|
			primary_svm_data_lifs.concat(svm_get_data_lifs("primary",protocol))
		}
=end # for future use

		secondary_svm_data_lifs = []
		data_protocols = ["nfs","cifs","iscsi","fcp","fcache"]
		data_protocols.each{ |protocol|
			secondary_svm_data_lifs.concat(svm_get_data_lifs("secondary",protocol))
		}
		@secondary_vsm_properties["data_lifs"] = secondary_svm_data_lifs
		
		#get luns from primary
		@primary_vsm_properties["luns"] = svm_get_luns_primary
		
		else # if isfirst then 
			
			# @primary_vsm_properties["luns"] = svm_get_luns_primary if check_connection_to_netapp("primary_cluster", "Refreshing luns on primary SVM")

			##### ===== @primary_vsm_properties["luns"] = svm_get_luns_primary if check_connection_to_netapp("primary_cluster", "Refreshing luns on primary SVM")
			if check_connection_to_netapp("primary_cluster", "Refreshing luns of primary SVM") then
				#puts "sleeping for 10 sec: luns"
				#sleep 10
				#puts "end sleeping"
				tmp = svm_get_luns_primary
				if ! tmp.class.name.eql?("NilClass") then
					@primary_vsm_properties["luns"] = tmp
				else #! tmp.class.name.eql?("NilClass") then

				end # ! tmp.class.name.eql?("NilClass") then
			end
			##### /===== @primary_vsm_properties["luns"] = svm_get_luns_primary if check_connection_to_netapp("primary_cluster", "Refreshing luns on primary SVM")

			## periodical data lif checking
			if check_connection_to_netapp("secondary_cluster", "Refreshing data lifs of secondary SVM") then
				#puts "sleeping for 10 sec: data lifs"
				#sleep 10
				#puts "end sleeping"
				
				secondary_svm_data_lifs = []
				data_protocols = ["nfs","cifs","iscsi","fcp","fcache"]
				tmp_error = false
				data_protocols.each{ |protocol|
					if ! tmp_error	then
						tmp = svm_get_data_lifs("secondary",protocol)
						if ! tmp.class.name.eql?("NilClass") then
							secondary_svm_data_lifs.concat(tmp)
						else # if ! tmp.class.name.eql?("NilClass") then
							tmp_error = true
						end # if ! tmp.class.name.eql?("NilClass") then
					end # if ! tmp_error	then
				}
				@secondary_vsm_properties["data_lifs"] = secondary_svm_data_lifs if ! tmp_error
			end	# if check_connection_to_netapp("secondary_cluster", "Refreshing data lifs from secondary SVM") then		
			## data lif checking

		end



	end # fillup_svm_details(isfirst)



end # end class

